@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP                   #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE TypeFamilies          #-}
@@ -29,6 +30,13 @@ module Diagrams.Animation
 
        ) where
 
+import           Data.Active
+#if __GLASGOW_HASKELL__ < 710
+import           Control.Applicative       ((<$>))
+import           Data.Foldable             (foldMap)
+#endif
+import           Data.Semigroup
+
 import           Diagrams.Core
 
 import           Diagrams.Animation.Active ()
@@ -38,18 +46,13 @@ import           Diagrams.TrailLike
 import           Diagrams.TwoD.Shapes
 import           Diagrams.TwoD.Types
 
-import           Data.Active
-import           Data.Semigroup
-
-import           Control.Applicative       ((<$>))
-import           Data.Foldable             (foldMap)
-import           Data.VectorSpace
+import           Linear.Metric
 
 -- | A value of type @QAnimation b v m@ is an animation (a
 --   time-varying diagram with start and end times) that can be
 --   rendered by backspace @b@, with vector space @v@ and monoidal
 --   annotations of type @m@.
-type QAnimation b v m = Active (QDiagram b v m)
+type QAnimation b v n m = Active (QDiagram b v n m)
 
 -- | A value of type @Animation b v@ is an animation (a time-varying
 --   diagram with start and end times) in vector space @v@ that can be
@@ -58,7 +61,7 @@ type QAnimation b v m = Active (QDiagram b v m)
 --   Note that @Animation@ is actually a synonym for @QAnimation@
 --   where the type of the monoidal annotations has been fixed to
 --   'Any' (the default).
-type Animation b v = QAnimation b v Any
+type Animation b v n = QAnimation b v n Any
 
 -- $animComb
 -- Most combinators for working with animations are to be found in the
@@ -90,16 +93,16 @@ type Animation b v = QAnimation b v Any
 --
 --   See also 'animRect' for help constructing a background to go
 --   behind an animation.
-animEnvelope :: (Backend b v, OrderedField (Scalar v), InnerSpace v, Monoid' m)
-           => QAnimation b v m -> QAnimation b v m
+animEnvelope :: (Backend b v n, OrderedField n, Metric v, Monoid' m)
+           => QAnimation b v n m -> QAnimation b v n m
 animEnvelope = animEnvelope' 30
 
 -- | Like 'animEnvelope', but with an adjustible sample rate.  The first
 --   parameter is the number of samples per time unit to use.  Lower
 --   rates will be faster but less accurate; higher rates are more
 --   accurate but slower.
-animEnvelope' :: (Backend b v, OrderedField (Scalar v), InnerSpace v, Monoid' m)
-            => Rational -> QAnimation b v m -> QAnimation b v m
+animEnvelope' :: (Backend b v n, OrderedField n, Metric v, Monoid' m)
+            => Rational -> QAnimation b v n m -> QAnimation b v n m
 animEnvelope' r a = withEnvelope (simulate r a) <$> a
 
 -- | @animRect@ works similarly to 'animEnvelope' for 2D diagrams, but
@@ -109,18 +112,16 @@ animEnvelope' r a = withEnvelope (simulate r a) <$> a
 --
 --   Uses 30 samples per time unit by default; to adjust this number
 --   see 'animRect''.
-animRect :: (TrailLike t, Enveloped t, Transformable t, Monoid t, V t ~ R2
-            , Monoid' m)
-         => QAnimation b R2 m -> t
+animRect :: (InSpace V2 n t, Num n, Monoid' m, TrailLike t, Enveloped t, Transformable t, Monoid t)
+         => QAnimation b V2 n m -> t
 animRect = animRect' 30
 
 -- | Like 'animRect', but with an adjustible sample rate.  The first
 --   parameter is the number of samples per time unit to use.  Lower
 --   rates will be faster but less accurate; higher rates are more
 --   accurate but slower.
-animRect' :: (TrailLike t, Enveloped t, Transformable t, Monoid t, V t ~ R2
-             , Monoid' m)
-          => Rational -> QAnimation b R2 m -> t
+animRect' :: (InSpace V2 n t, Num n, Monoid' m, TrailLike t, Enveloped t, Transformable t, Monoid t)
+          => Rational -> QAnimation b V2 n m -> t
 animRect' r anim
     | null results = rect 1 1
     | otherwise    = boxFit (foldMap boundingBox results) (rect 1 1)
